package com.huimv.eartag2.api.config;

import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * 数据库定时备份任务
 * 在backup文件夹中备份最近七日的数据库文件, 备份文件夹 与当前程序同一目录
 */
@Component
@Slf4j
public class DbJob {

    private final Environment environment;

    public DbJob(Environment environment) {
        this.environment = environment;
    }

    /**
     * 定时时间是每天凌晨5点。
     */
    @Scheduled(cron = "0 0 5 * * ?")
    public void backup() throws IOException {
        LocalDateTime now = LocalDateTime.now();
        
        log.info("*******************时间：【{}】, 系统开启定时任务数据库备份*******************", now);
        
		// 数据库配置信息
        String user = environment.getProperty("spring.datasource.druid.username");

        String password = environment.getProperty("spring.datasource.druid.password");

        String url = environment.getProperty("spring.datasource.druid.url");

        // 第三个 ：号下标
        int subStrIndex = url.indexOf(":", url.indexOf(":", url.indexOf(":") + 1) + 1);

        // IP
        String host = url.substring(url.indexOf("//") + 2, subStrIndex);

        // 端口
        String subStr2 = url.substring(subStrIndex);
        String port = subStr2.substring(1, subStr2.indexOf("/"));

        // 数据库名
        String dataBaseName = subStr2.substring(subStr2.indexOf("/") + 1, subStr2.indexOf("?"));

        // 环境
        String os = System.getProperties().getProperty("os.name");

        log.info("备份环境信息：【{}】, 用户名：【{}】,密码：【{}】, 地址：【{}】, 端口：【{}】,数据库：【{}】", os, url, password, host, port, dataBaseName);


        LocalDate localDate = LocalDate.now();

        String fileName = localDate + ".sql";

        File file = new File("backup", fileName);

        file.getParentFile().mkdirs();

        file.createNewFile();

        // 备份今天数据库
        DbUtil.backup("10.0.0.77", "3309", user, password, dataBaseName, file);
        //备份->对历史数据进行冷处理，减轻数据库压力
        DbUtil.load(file,user,password,dataBaseName);
        // 删除七天前数据库备份文件 LocalDate
        LocalDate before = localDate.minusDays(7);
        String fileBeforeName = before + ".sql";
        File fileBefore = new File("backup", fileBeforeName);
        if (fileBefore.exists()) {
            fileBefore.delete();
        }
        log.info("*******************时间：【{}】, 系统结束定时任务数据库备份*******************", now);
    }

}

