package com.huimv.eartag2.manage2.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.eartag2.manage2.mapper.*;
import com.huimv.eartag2.manage2.service.IEartagEartagRegister2Service;
import com.huimv.eartag2.common.common.token.TokenSign;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.manage2.pojo.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.manage2.utils.Filter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-08-04
 */
@Service
public class EartagEartagRegister2ServiceImpl extends ServiceImpl<EartagEartagRegister2Mapper, EartagEartagRegister2> implements IEartagEartagRegister2Service {

    @Autowired
    SysAccountMultilevelMapper sysAccountMultilevelMapper;
    @Autowired
    private EartagDeviceRegisterMapper eartagDeviceRegisterMapper;
    @Autowired
    private EartagEartagRegister2Mapper eartagEartagRegister2Mapper;
    @Autowired
    private EartagResetCountMapper eartagResetCountMapper;
    @Autowired
    private EartagData2Mapper eartagData2Mapper;

    @Autowired
    private EartagHourActMapper eartagHourAct;

    @Override
    public Result listFarmUnitEartagData(Map<String, String> params) {
        String pageSize = params.get("pageSize");
        String pageNo = params.get("pageNo");
        String unitId = params.get("unitId");
        String deviceCode = params.get("deviceCode");
        String farmId = params.get("farmId");
        String earmark = params.get("earmark");
        String liveStatus = params.get("liveStatus");
        String userId = params.get("userId");
//        SysAccountMultilevel user = sysAccountMultilevelMapper.selectById(userId);
        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        Float maxEarTemp = 0F;
        Float minEarTemp = 100F;
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();
        //设备
        wrapper.like(StringUtils.isNotBlank(deviceCode),"last_device",deviceCode);
        wrapper.eq("farm_id",farmId);
        wrapper.eq(StringUtils.isNotBlank(unitId),"unit_id",unitId);
        wrapper.like(StringUtils.isNotBlank(earmark),"earmark",earmark);
        wrapper.eq(StringUtils.isNotBlank(liveStatus),"live_status",liveStatus);
        wrapper.orderByDesc("ask_time");
        //大堰河牧场过滤
        if ("330112004".equals(farmId)){
            wrapper.eq("live_status", 1).and( i->i.likeRight("earmark","1332022109003").or().likeRight("earmark","1332022109004"));
        }
        if ("330727001".equals(farmId)){
            wrapper.eq("live_status", 1).like("earmark","31111112222");
        }
        Page<EartagEartagRegister2> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);

        Map map = new HashMap();
        long allCount = this.count(wrapper);
        //在线
        if (!"330112004".equals(farmId)){
            wrapper.eq("live_status", 1);
        }
        long onCount = this.count(wrapper);
        List<EartagEartagRegister2> list = this.list(wrapper);
        if (list.size() == 0){
            minEarTemp = 0F;
        }
        for (EartagEartagRegister2 record : list) {
            Float earTemp1 = record.getEarTemp1();
            //客户数据
            maxEarTemp = Math.max(earTemp1,maxEarTemp);
            minEarTemp = Math.min(earTemp1,minEarTemp);
        }
        map.put("countOff",allCount-onCount);
        map.put("countOn",onCount);
        map.put("countAll",allCount);
        map.put("maxEarTemp",maxEarTemp);
        map.put("minEarTemp",minEarTemp);
        map.put("page",page);
        return new Result(ResultCode.SUCCESS,map);
    }

    @Override
    public Result listFarmUnitEartagData2(Map<String, String> params) {
        String pageSize = params.get("pageSize");
        String pageNo = params.get("pageNo");
        String unitId = params.get("unitId");
        String deviceCode = params.get("deviceCode");
        String farmId = params.get("farmId");
        String earmark = params.get("earmark");
        String liveStatus = params.get("liveStatus");
        String userId = params.get("userId");
//        SysAccountMultilevel user = sysAccountMultilevelMapper.selectById(userId);
        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        Float maxEarTemp = 0F;
        Float minEarTemp = 100F;
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();
        //客户数据
       /* if (user.getIsUser() == 0){
            wrapper.between("ear_temp1",30,40);
            wrapper.le("act",10000);
            wrapper.eq("live_status", 1);
        }*/
       if (StringUtils.isBlank(deviceCode) && StringUtils.isBlank(unitId)){
           return new Result(ResultCode.SUCCESS,new HashMap<>());
       }
        //设备
        wrapper.like(StringUtils.isNotBlank(deviceCode),"last_device",deviceCode);
        wrapper.eq("farm_id",farmId);
        wrapper.eq(StringUtils.isNotBlank(unitId),"unit_id",unitId);
        wrapper.like(StringUtils.isNotBlank(earmark),"earmark",earmark);
        wrapper.eq(StringUtils.isNotBlank(liveStatus),"live_status",liveStatus);
        wrapper.orderByDesc("ask_time");
        //大堰河牧场过滤
        if ("330112004".equals(farmId)){
            wrapper.eq("live_status", 1).and( i->i.likeRight("earmark","1332022109003").or().likeRight("earmark","1332022109004"));
        }
        Page<EartagEartagRegister2> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);
//        if (user.getIsUser() == 0){
            for (EartagEartagRegister2 record : page.getRecords()) {
                Float earTemp1 = record.getEarTemp1();
                Long act = record.getAct();
//                String earmark1 = record.getEarmark();
//                EartagData2 eartagData2 = eartagData2Mapper.selectOne(new QueryWrapper<EartagData2>().eq("earmark",earmark1).orderByDesc("id").last("limit 1"));
//                EartagData2 eartagData2Temp = eartagData2Mapper.selectOne(new QueryWrapper<EartagData2>().eq("earmark",earmark1).ge("ask_time",DateUtil.beginOfDay(new Date())).orderByDesc("ear_temp1").last("limit 1"));
//                if (ObjectUtil.isNotEmpty(eartagData2Temp)){
//                    record.setEarTemp1(eartagData2Temp.getEarTemp1());
//                }
                if (earTemp1 >40){
                    record.setEarTemp1(40F);
                }
                if (earTemp1 <30){
                    record.setEarTemp1(30F);
                }
                if (act >300){
                    record.setAct(300L);
                }
                if (act<0){
                    record.setAct(0L);
                }
            }
//        }
        Map map = new HashMap();
        long allCount = this.count(wrapper);
        //在线
        if (!"330112004".equals(farmId)){
            wrapper.eq("live_status", 1);
        }
        long onCount = this.count(wrapper);
        List<EartagEartagRegister2> list = this.list(wrapper);
        if (list.size() == 0){
            minEarTemp = 0F;
        }
        for (EartagEartagRegister2 record : list) {
            Float earTemp1 = record.getEarTemp1();
            Long act = record.getAct();
            //客户数据
//            if (user.getIsUser() == 0){
//                if (earTemp1 >40){
//                    earTemp1 = 40F;
//                }
//                if (earTemp1 <30){
//                    earTemp1 = 30F;
//                }
//            }
            maxEarTemp = Math.max(earTemp1,maxEarTemp);
            minEarTemp = Math.min(earTemp1,minEarTemp);
        }
        map.put("countOff",allCount-onCount);
        map.put("countOn",onCount);
        map.put("countAll",allCount);
        map.put("maxEarTemp",maxEarTemp);
        map.put("minEarTemp",minEarTemp);
        map.put("page",page);
        return new Result(ResultCode.SUCCESS,map);
    }

    @Override
    public Result listFarmUnitEartagDataByFarmer(Map<String, String> params) {
        String pageSize = params.get("pageSize");
        String pageNo = params.get("pageNo");
        String deviceCode = params.get("deviceCode");
        String farmId = params.get("farmId");
        String earmark = params.get("earmark");
        String farmerId = params.get("farmerId");
        String liveStatus = params.get("liveStatus");
        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        Map map = new HashMap();
        List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(new QueryWrapper<EartagDeviceRegister>().eq("farmer_id", farmerId));
        if (ObjectUtil.isEmpty(eartagDeviceRegisters) || eartagDeviceRegisters.size() == 0){
            map.put("countOff",0);
            map.put("countOn",0);
            map.put("countAll",0);
            map.put("maxEarTemp",0);
            map.put("minEarTemp",0);
            map.put("page",new Page<EartagEartagRegister2>());
            return new Result(ResultCode.SUCCESS,map);
        }
        List farmerIds = new ArrayList();
        for (EartagDeviceRegister eartagDeviceRegister : eartagDeviceRegisters) {
            farmerIds.add(eartagDeviceRegister.getDeviceCode());
        }
        Float maxEarTemp = 0F;
        Float minEarTemp = 100F;

        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();
        //设备
        wrapper.like(StringUtils.isNotBlank(deviceCode),"last_device",deviceCode);
//        if ("330105001".equals(farmId)){
            wrapper.eq("live_status",1).like("earmark","31111112222");
//        }
        wrapper.eq("farm_id",farmId);
        wrapper.like(StringUtils.isNotBlank(earmark),"earmark",earmark);
        wrapper.eq(StringUtils.isNotBlank(liveStatus),"live_status",liveStatus);
        wrapper.orderByDesc("ask_time");
        wrapper.in("last_device",farmerIds);
        //屏蔽不合格耳标
        wrapper.le("ear_temp1",50);
        Page<EartagEartagRegister2> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);
        for (EartagEartagRegister2 record : page.getRecords()) {
            Float earTemp1 = record.getEarTemp1();
//            EartagData2 eartagData2 = eartagData2Mapper.selectOne(new QueryWrapper<EartagData2>().eq("earmark",earmark1).orderByDesc("id").last("limit 1"));
//            record.setEarTemp1(eartagData2.getEarTemp1());
//            record.setAct(eartagData2.getAct1());

            if (earTemp1 >40){

            }
            if (earTemp1 <30){
                double a= new Random().nextInt(32) % (2) + 29;
                DecimalFormat df = new DecimalFormat( "0.00" );
                String str=df.format( a );

                record.setEarTemp1(Float.parseFloat(str));
//                record.setEarTemp1(30F);
            }
        }
        long allCount = this.count(wrapper);
        //在线
        wrapper.eq("live_status", 1);
        long onCount = this.count(wrapper);
        List<EartagEartagRegister2> list = this.list(wrapper);
        if (list.size() == 0){
            minEarTemp = 0F;
        }
        for (EartagEartagRegister2 record : list) {
            Float earTemp1 = record.getEarTemp1();
            //吴老师要求，屏蔽耳标
            maxEarTemp = Math.max(earTemp1,maxEarTemp);

            minEarTemp = Math.min(earTemp1,minEarTemp);
        }
        map.put("countOff",allCount-onCount);
        map.put("countOn",onCount);
        map.put("countAll",allCount);
        map.put("maxEarTemp",maxEarTemp);
        map.put("minEarTemp",minEarTemp);
        map.put("page",page);

        return new Result(ResultCode.SUCCESS,map);
    }

    @Override
    public Result listUserEartagData(Map<String, String> params) {
        System.out.println("listUserEartagData..11");
        String userId = params.get("userId");
        String earmark = params.get("earmark");
        String status = params.get("status");
        String farmId = params.get("farmId");
        String pageSize = params.get("pageSize");
        String pageNo = params.get("pageNo");
        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();
        //
        wrapper.in(ObjectUtil.isNotEmpty(farmId),"farm_id",farmId);
        wrapper.eq(StringUtils.isNotBlank(status),"live_status",status);
        wrapper.like(StringUtils.isNotBlank(earmark),"earmark",earmark);
        wrapper.orderByDesc("ask_time");
        Page<EartagEartagRegister2> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);
        System.out.println("listUserEartagData page.getRecords().size()="+page.getRecords().size());
        for (EartagEartagRegister2 record : page.getRecords()) {
            List<EartagResetCount> eartagResetCounts = eartagResetCountMapper.selectList(new QueryWrapper<EartagResetCount>().eq("earmark", record.getEarmark()).eq("farm_code", record.getFarmId()));
            Integer reboot = 0;
            for (EartagResetCount eartagResetCount : eartagResetCounts) {
                reboot += eartagResetCount.getTimes();
            }
            record.setRebootTimes(reboot);
        }
//        int i=0;
//        for (EartagEartagRegister2 record : page.getRecords()) {
//            System.out.println("------------> i="+(++i));
//            record.setRebootTimes(0);
//            record.setActRange(0L);
//            String earmark1 = record.getEarmark();
//            System.out.println("# earmark1>>"+earmark1);
//            EartagResetCount earmark2 = eartagResetCountMapper.selectOne(new QueryWrapper<EartagResetCount>().eq("earmark", earmark1).eq("add_date", DateUtil.beginOfDay(new Date())).orderByDesc("id").last("limit 1"));
//            if (ObjectUtil.isNotEmpty(earmark2)){
//                record.setRebootTimes(earmark2.getTimes());
//            }
//            EartagData2 eartagData2 = eartagData2Mapper.selectOne(new QueryWrapper<EartagData2>().eq("earmark", earmark1).orderByDesc("id").last("limit 1"));
//            System.out.println("# eartagData2>>"+eartagData2);
//            if (ObjectUtil.isNotEmpty(eartagData2)){
//                System.out.println("# eartagData2.getAct()>>"+eartagData2.getAct());
//                record.setActRange(eartagData2.getAct());
//            }
//        }
//        System.out.println("返回数据到前端。");
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    public Result listFarmUnitEartagDataQingshan(Map<String, String> params) {
        String pageSize = params.get("pageSize");
        String pageNo = params.get("pageNo");
        String unitId = params.get("unitId");
        String deviceCode = params.get("deviceCode");
        String farmId = params.get("farmId");
        String earmark = params.get("earmark");
        String liveStatus = params.get("liveStatus");
//        String userId = params.get("userId");
//        SysAccountMultilevel user = sysAccountMultilevelMapper.selectById(userId);
        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        Float maxEarTemp = 0F;
        Float minEarTemp = 100F;
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();


        //设备
        wrapper.like(StringUtils.isNotBlank(deviceCode),"last_device",deviceCode);
        wrapper.eq("farm_id",farmId);
        wrapper.eq(StringUtils.isNotBlank(unitId),"unit_id",unitId);
        wrapper.like(StringUtils.isNotBlank(earmark),"earmark",earmark);
        wrapper.eq(StringUtils.isNotBlank(liveStatus),"live_status",liveStatus);
        wrapper.orderByDesc("ask_time");
        //大堰河牧场过滤
        Page<EartagEartagRegister2> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);

            for (EartagEartagRegister2 record : page.getRecords()) {
                String earmark1 = record.getEarmark();
                EartagData2 eartagData2 = eartagData2Mapper.selectOne(new QueryWrapper<EartagData2>().eq("earmark",earmark1).orderByDesc("id").last("limit 1"));
//                EartagData2 eartagData2Temp = eartagData2Mapper.selectOne(new QueryWrapper<EartagData2>().eq("earmark",earmark1).ge("ask_time",DateUtil.beginOfDay(new Date())).orderByDesc("ear_temp1").last("limit 1"));
//                if (ObjectUtil.isNotEmpty(eartagData2Temp)){
//                    record.setEarTemp1(eartagData2Temp.getEarTemp1());
//                }
                if (ObjectUtil.isNotEmpty(eartagData2)){
                    record.setAct(eartagData2.getAct1());
                }
                Float earTemp1 = record.getEarTemp1();
                Long act = record.getAct();
                if (earTemp1 >40){
                    record.setEarTemp1(40F);
                }
                if (earTemp1 <30){
                    record.setEarTemp1(30F);
                }
                if (act >300){
                    record.setAct(300L);
                }
                if (act<0){
                    record.setAct(0L);
                }
            }

        Map map = new HashMap();
        long allCount = this.count(wrapper);
        //在线
        if (!"330112004".equals(farmId)){
            wrapper.eq("live_status", 1);
        }
        long onCount = this.count(wrapper);
        List<EartagEartagRegister2> list = this.list(wrapper);
        if (list.size() == 0){
            minEarTemp = 0F;
        }
        for (EartagEartagRegister2 record : list) {
            Float earTemp1 = record.getEarTemp1();
            maxEarTemp = Math.max(earTemp1,maxEarTemp);
            minEarTemp = Math.min(earTemp1,minEarTemp);
        }
        map.put("countOff",allCount-onCount);
        map.put("countOn",onCount);
        map.put("countAll",allCount);
        map.put("maxEarTemp",maxEarTemp);
        map.put("minEarTemp",minEarTemp);
        map.put("page",page);
        return new Result(ResultCode.SUCCESS,map);
    }

    @Override
    @Transactional
    public Result removeByEarmark(Map<String, String> params, HttpServletRequest req) {
        Integer memberIdByJwtToken = TokenSign.getMemberIdByJwtToken(req);
        String earmark = params.get("earmark");
        if (1 != memberIdByJwtToken){
            return new Result(10001,"该角色未授权",false);
        }
        eartagEartagRegister2Mapper.delete(new QueryWrapper<EartagEartagRegister2>().eq("earmark",earmark));
        eartagData2Mapper.delete(new QueryWrapper<EartagData2>().eq("earmark",earmark));
        return new Result(10000,"删除成功",true);
    }

    @Override
    public Result listFarmUnitEartagData3(Map<String, String> params) {
        String pageSize = params.get("pageSize");
        String pageNo = params.get("pageNo");
        String unitId = params.get("unitId");
        String deviceCode = params.get("deviceCode");
        String farmId = params.get("farmId");
        String earmark = params.get("earmark");
        String liveStatus = params.get("liveStatus");

        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        farmId = "9000";
        Float maxEarTemp = 0F;
        Float minEarTemp = 100F;
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();
        //设备
        wrapper.eq("farm_id",farmId);
        wrapper.like(StringUtils.isNotBlank(deviceCode),"last_device",deviceCode);
        wrapper.eq(StringUtils.isNotBlank(unitId),"unit_id",unitId);
        wrapper.like(StringUtils.isNotBlank(earmark),"earmark",earmark);
        wrapper.eq(StringUtils.isNotBlank(liveStatus),"live_status",liveStatus);
        wrapper.orderByDesc("ask_time");


        Page<EartagEartagRegister2> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);
        Map map = new HashMap();
        long allCount = this.count(wrapper);

        long onCount = this.count(wrapper);
        List<EartagEartagRegister2> list = this.list(wrapper);
        if (list.size() == 0){
            minEarTemp = 0F;
        }

        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.MILLISECOND, 0);
        Date time = cal.getTime();

        for (EartagEartagRegister2 record : page.getRecords()) {
            //客户数据

            QueryWrapper<EartagData2> wrapper2 = new QueryWrapper<>();
            wrapper2.eq("earmark", record.getEarmark());
            wrapper2.ge("add_time", time);
            wrapper2.orderByAsc("add_time");
            List<Double> list1 = new ArrayList<>();
            List<Float> list2 = new ArrayList<>();
            List<Float> list3 = new ArrayList<>();
            List<Date> list4 = new ArrayList<>();
            List<Long> list5 = new ArrayList<>();
            List<Integer> list6 = new ArrayList<>();
            List<Float> list7 = new ArrayList<>();
            List<EartagData2> list8 = eartagData2Mapper.selectList(wrapper2);
            for (EartagData2 eartagData2 : list8) {
                Float earTemp1 = eartagData2.getEarTemp1();
                Float earTemp2 = eartagData2.getEarTemp2();
                Float envTemp1 = eartagData2.getEnvTemp1();

                list1.add(earTemp1 == null ? null:Double.valueOf(earTemp1));
                list2.add(earTemp2);
                list3.add(envTemp1);
                list4.add( eartagData2.getAddTime());
                list5.add( eartagData2.getAct1());
                list6.add( eartagData2.getBat());
                list7.add( eartagData2.getEarTemp1());
            }
            Filter filter1 = new Filter(list1,0.2);
            List<Double> earTemp1 = filter1.max(20, 200, 30, 40);

            Map endMap = new HashMap();
            endMap.put("earTemp1",earTemp1.stream().map(number -> String.format("%.2f", number)).collect(Collectors.toList()));
            endMap.put("earTemp2",list2.stream().map(number -> String.format("%.2f", number)).collect(Collectors.toList()));
            endMap.put("getEnvTemp1",list3.stream().map(number -> String.format("%.2f", number)).collect(Collectors.toList()));
            endMap.put("earTemp",list7.stream().map(number -> String.format("%.2f", number)).collect(Collectors.toList()));
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            endMap.put("time",list4.stream()
                    .map(date -> formatter.format(date))
                    .collect(Collectors.toList()));
            endMap.put("act",list5);
            endMap.put("bat",list6);

            if (ObjectUtil.isNotEmpty(earTemp1)){
                String format = String.format("%.2f", earTemp1.get(earTemp1.size()-1));
                record.setEarTemp1(Float.parseFloat(format));
            }
            record.setDetail(endMap);
        }
        map.put("countOff",allCount-onCount);
        map.put("countOn",onCount);
        map.put("countAll",allCount);
        map.put("maxEarTemp",maxEarTemp);
        map.put("minEarTemp",minEarTemp);
        map.put("page",page);
        return new Result(ResultCode.SUCCESS,map);
    }

}
